searchData={"items":[{"type":"module","title":"FlameEC2","doc":"A `FLAME.Backend` using [AWS EC2](https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/concepts.html) machines.","ref":"FlameEC2.html"},{"type":"module","title":"How Does It Work? - FlameEC2","doc":"`FlameEC2` loads a release bundle that is stored in S3, and runs this release on a child node that it raises.\nThe release will always be run with the built in release start command, with the environment marking it as a FLAME child.\n\nTo facilitate shutdown of the node, `FlameEC2` runs the release under a systemd service, which is installed with a Post Stop hook\nthat will automatically run a shutdown command on the instance when the release is stopped. This, in combination with setting the\n`\"InstanceInitiatedShutdownBehavior\"` to `\"terminate\"`, guarantees that when the release is stopped, the instance will trigger its\nshutdown and termination behavior.","ref":"FlameEC2.html#module-how-does-it-work"},{"type":"module","title":"Usage - FlameEC2","doc":"To use, you must tell FLAME to use the `FlameEC2` backend by default.\nThis can be set via application configuration in your `config/runtime.exs` withing a `:prod` block:\n\n```elixir\nif config_env() == :prod do\n  config :flame, :backend, FlameEC2\n  ...\nend\n```\n\nYou must ensure that the IAM Role of whatever is running this backend has access to all of the necessary APIs.\nThe following is a minimal IAM Policy recommended to allow the backend to access all required APIs:\n\n```json\n{\n    \"Version\": \"2012-10-17\",\n    \"Statement\": [\n        {\n            \"Sid\": \"ec2RunInstances\",\n            \"Effect\": \"Allow\",\n            \"Action\": [\n                \"ec2:DescribeTags\",\n                \"ec2:CreateTags\",\n                \"ec2:DeleteTags\",\n                \"ec2:RunInstances\"\n            ],\n            \"Resource\": \"*\"\n        },\n        {\n            \"Sid\": \"ssmParameters\",\n            \"Effect\": \"Allow\",\n            \"Action\": [\n                \"ssm:GetParameters\"\n            ],\n            \"Resource\": \"*\"\n        },\n        {\n            \"Sid\": \"iamRolePassing\",\n            \"Effect\": \"Allow\",\n            \"Action\": [\n                \"iam:PassRole\"\n            ],\n            \"Resource\": [\n                \"arn:aws:iam::*:instance-profile/*\",\n                \"arn:aws:iam::*:role/*\"\n            ],\n            \"Condition\": {\n                \"StringEquals\": {\n                    \"iam:PassedToService\": \"ec2.amazonaws.com\"\n                }\n            }\n        },\n        {\n            \"Sid\": \"s3GetRelease\",\n            \"Effect\": \"Allow\",\n            \"Action\": [\n                \"s3:ListBucket\",\n                \"s3:GetObject\"\n            ],\n            \"Resource\": \"*\"\n        }\n    ]\n}\n```","ref":"FlameEC2.html#module-usage"},{"type":"module","title":"Configuration - FlameEC2","doc":"Configuration of the backend mirrors a subset of the\n[AWS EC2 LaunchInstance API](https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_RunInstances.html) options,\nto allow us to raise a machine properly.\n\nAll of the configurations may be automatically set to the current instance's details by setting `auto_configure` to true:\n\n```elixir\nconfig :flame, FlameEC2, auto_configure: true\n...\n```\n\nIf you'd like to manually configure the details in your pool, you can read about the available configurations below.","ref":"FlameEC2.html#module-configuration"},{"type":"module","title":"Required Configurations - FlameEC2","doc":"* `:s3_bundle_url` - The S3 URL of your release bundle.\nThis value must be set in order to ensure that we can start the application properly on the child node.\nThe bundle is expected to either be a directory that will be synced locally onto the child node,\nor a `.tar.gz` file that will be copied to the node and decompressed.\n\n* `:app` - The name of your application. Defaults to `System.get_env(\"RELEASE_NAME\")`.\nIf this environment variable is not set for some reason, this value must be set in the configuration.\n\n* `:image_id` - The ID of the AMI to use. This is a required attribute if you are not using a launch template, and has no default.\n\n* `:launch_template_id` - The ID of the Launch Template to use. This is a required attribute if you are not using an image id, and has no default.\n\n* `:launch_template_version` - The version number of the launch template, or \"$Default\", or \"$Latest\". Defaults to \"$Default\".\n\n* `:subnet_id` - The subnet ID for the attached network interface. This is required in order to properly communicate with the instance.\n\n* `:security_group_id` - The security group ID for the attached network interface. This is required in order to properly communicate with the instance.","ref":"FlameEC2.html#module-required-configurations"},{"type":"module","title":"Optional Configurations - FlameEC2","doc":"* `:boot_timeout` - A timeout for booting a new node. Defaults to 120_000 (2 minutes).\n**Note!** The boot timeout must account for things like the actual initialization of an instance.\nInstances can take a couple of minutes to move from \"Pending\" to \"Running\" depending on capacity,\nand that doesn't include the actual instance initialization and the script that needs to run to set\nup the release on the machine. When using FlameEC2, you may want to consider setting this value to something\nthat is higher to avoid timeout issues due to slowness of machine startups, or setting the pool minimum to 1\nto ensure that you have a \"warm\" pool of instances that can run tasks.\n\n* `:instance_type` - The instance type that should be used. Defaults to \"t3.nano\", which falls under the AWS free tier,\nhowever, you likely want to change this to something that's more appropriate for your pool's workload.\n\n* `:iam_instance_profile` - The ARN of the instance profile to assign to this machine.\nYou must ensure that this instance profile is set to a profile which can access whatever your FLAME nodes will need.\nWhen auto-configured, it will be set to the current machine's instance profile, however, in some cases this profile may\nbe too broad and you may want to create a role specific to the FLAME nodes.\n\n* `:key_name` - The name of the key pair to use for you to be able to connect to the instance.\nThis is not required, however, an instance that was created without a key pair will be inaccessible without another way to log in.\n\n* `:instance_metadata_url` - The EC2 instance metadata URL. This is used when auto-configuring the pool with the `auto_configure` configuration set to `true`.\nDefaults to \"http://169.254.169.254/latest/meta-data/\" (note the trailing slash), which is the internal EC2 metadata URL.\nThis can be adjusted for local testing, but likely does not need to be adjusted outside of this use case.\n\n* `:instance_metadata_token_url` - The EC2 instance metadata token URL. This is used when auto-configuring the pool with the `auto_configure` configuration set to `true`.\nDefaults to \"http://169.254.169.254/latest/api/token\", which is the internal EC2 metadata token URL.\nThis can be adjusted for local testing, but likely does not need to be adjusted outside of this use case.\n\n* `:ec2_service_endpoint` - The URL of the EC2. Defaults to \"https://ec2.amazonaws.com/\", which is the AWS EC2 endpoint.\nThis can be adjusted for local testing, but likely does not need to be adjusted outside of this use case.","ref":"FlameEC2.html#module-optional-configurations"},{"type":"module","title":"Environment Variables - FlameEC2","doc":"The FLAME EC2 machines *do not* inherit the environment variables of the parent.\nYou must explicit provide the environment that you would like to forward to the\nmachine. For example, if your FLAME's are starting your Ecto repos, you can copy\nthe env from the parent:\n\n```elixir\nconfig :flame, FlameEC2,\n  env: %{\n    \"DATABASE_URL\" => System.fetch_env!(\"DATABASE_URL\"),\n    \"POOL_SIZE\" => \"1\"\n  }\n```\n\nOr pass the env to each pool:\n\n```elixir\n{FLAME.Pool,\n  name: MyRunner,\n  backend: {FlameEC2, env: %{\"DATABASE_URL\" => System.fetch_env!(\"DATABASE_URL\")}}\n}\n```","ref":"FlameEC2.html#module-environment-variables"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThis changelog follows the same style that I have seen LiveView, Phoenix, and Elixir use in the past. I'll try to make sure that I maintain it - probably should create some sort of automated process for it... who knows. For now - there's only one release so this should be good enough!","ref":"changelog.html"},{"type":"extras","title":"1.0.0 - Changelog","doc":"This is the first official release! So everything is empty. Read the docs to get started - have fun!","ref":"changelog.html#1-0-0"},{"type":"extras","title":"Bug fixes - Changelog","doc":"","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"","ref":"changelog.html#enhancements"},{"type":"extras","title":"Deprecations - Changelog","doc":"","ref":"changelog.html#deprecations"},{"type":"extras","title":"Removal of previously deprecated functionality - Changelog","doc":"","ref":"changelog.html#removal-of-previously-deprecated-functionality"},{"type":"extras","title":"README","doc":"# FlameEC2\n\nA FLAME backend for EC2 Machines.","ref":"readme.html"},{"type":"extras","title":"Why not K8S/ECS/Docker/etc? - README","doc":"I don't like complicating my deployments too much - while I have used K8S and ECS and Docker Compose and have spent years learning their ins and outs, I've recently started preferring simple bare-metal style deployments. Just give me a VPS with SSH access, plus something like Ansible, CodeDeploy (I am using AWS after all), or just simple rsync and a systemd service and I'm happy.\n\nWith that in mind - I want my FLAME cluster to shine (see what I did there?) without needing all this fluff as well.","ref":"readme.html#why-not-k8s-ecs-docker-etc"},{"type":"extras","title":"Setup - README","doc":"See the moduledoc for `FlameEC2` for extensive documentation on different options.\n\nThe main thing that you need to set up properly is your `s3_bundle_url` value. This defines the bundle that will be loaded onto the runner machine, in order to run the application. There are various strategies to set up the release bundle url - for example, you may always push the latest release to a specific directory in S3 (the \"latest\" directory) and simply hardcode pointing to this URL. Alternatively, you may want to maintain versioining and use a Git tag or Git commit hash to point to the correct bundle - the version can be set in the build when compiling your release so that you can know which version to point to on any given release.","ref":"readme.html#setup"},{"type":"extras","title":"Usage Recommendation - README","doc":"","ref":"readme.html#usage-recommendation"},{"type":"extras","title":"FLAME.call Timeouts and FLAME.Pool Boot Timeouts - README","doc":"EC2 Machines can take some time coming up (even sometimes several minutes) due to how AWS manages capacity in their availability zones. The time is typically under 2 minutes, although it is highly dependent on the instance types that you are trying to raise. Because of this, the `:boot_timeout` for your `FLAME.Pool` and the `:timeout` option for `FLAME.call` should be set high enough to avoid timeouts happening due to waiting for an instance to move from Pending to Running.","ref":"readme.html#flame-call-timeouts-and-flame-pool-boot-timeouts"},{"type":"extras","title":"Warm Pool - README","doc":"If you want to ensure that you don't cause all of your function calls to take a long time due to waiting for capacity, it's highly recommended to have a warm pool (i.e. a pool with a minimum greater than 0). This will ensure that for initial calls, you will not wait for an instance to become ready. **Note:** While having a warm pool does ensure that you have an instance at the beginning, if your scale grows, you will eventually have to wait for new instances to boot up.","ref":"readme.html#warm-pool"},{"type":"extras","title":"Idle Runner Timeouts - README","doc":"In a similar vein to the above two recommendations - setting the `:idle_shutdown_after` option on your configured `FLAME.Pool` can help in ensuring that you do not scale down too quickly. The default value is set to `30_000` (30 seconds), but setting this to a higher value can help ensure that you aren't constantly raising and terminating instances.","ref":"readme.html#idle-runner-timeouts"},{"type":"extras","title":"Example Configuration - README","doc":"Based on the above recommendations, an example configuration that may work well would look something like this:\n\n```elixir\n{\n  FLAME.Pool,\n  name: FlameEC2Demo.Pool,\n  # Optionally, set the minimum to 1 so that we can run 100 concurrent tasks without worrying about waiting.\n  min: 0,\n  max: 10,\n  max_concurrency: 100,\n  # Set a global FLAME.call timeout that is higher than the boot timeout to ensure that we do not timeout calls before booting\n  timeout: 180_000,\n  # Set the boot timeout to a high enough number that we can have enough time to raise an instance\n  boot_timeout: 150_000,\n  # Set the idle shutdown timeout to high enough that we don't constantly raise and terminate instances\n  idle_shutdown_after: 60_000,\n}\n```","ref":"readme.html#example-configuration"},{"type":"extras","title":"Debugging Issues - README","doc":"The `FlameEC2` backend works by setting up an EC2 UserData script which will initialize the runner instance with a systemd service that is running the release bundle that you specify. If the runner is not able to start, there may be an issue during the start script which causes the whole process to fail. If there is a problem, you can see the logs of the start script by using the systemd journal as shown here:\n\n```sh\nsudo journalctl -t flame_ec2_init\n```\n\nThis should output any logs related to the start script to show whether or not there is a problem.","ref":"readme.html#debugging-issues"},{"type":"extras","title":"Installation - README","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `flame_ec2` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:flame_ec2, \"~> 0.0.1\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#installation"},{"type":"extras","title":"Contributing - README","doc":"Feel free to fork and make PRs! I'm definitely happy to have eyes on this and to get feedback so we can take care of edge cases.\n\nFor contributions, ensure that you run `git update-index --skip-worktree compose-env/ec2_mock_config.json`. The mock config gets reset\nand resynced in every `iex` session to ensure that the local compose environment is up to date for localized tests. This means that\nit is constantly changing, and therefore we should not track its changes. However, I do want to ensure that the file is distributed with\nthe repository, so that developers don't have to know to create it or have to do any setup before getting started.","ref":"readme.html#contributing"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}